// $Id: parameters-evaluate.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "parameters.h"
#include "djinniprocess.h"
#include "syntaxinfo.h"

#include <assert.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>

#include "checksyntax-command.hc"
#include "checksyntax-vserver.hc"
#include "checksyntax-int.hc"
#include "checksyntax-string.hc"
#include "checksyntax-range.hc"

static bool
iterateThroughParams(struct Parameters *self,
		     struct DjinniProcess const *proc,
		     char const *syntax,
		     unsigned int *code, char const **reason)
{
  char const	*ptr = syntax;
  size_t	i;
  bool		res = true;

  for (i=1; *ptr!='\0' && i<self->argc; ++i) {
    struct SyntaxInfo	info;
    if (!SyntaxInfo_init(&info, &ptr)) {
      *code   = 255;
      *reason = "Broken params-file";
      return false;
    }

    switch (info.type) {
      case synVSERVER	:
	res = (checkSyntax_vserver(self, self->argv[i], proc, reason) &&
	       checkSyntax_range(&info, true, self->argv[i], reason));
	break;
	
      case synSET	:
	res = checkSyntax_range(&info, false, self->argv[i], reason);
	break;

      case synINT	:
	res = checkSyntax_int(&info, self->argv[i], reason);
	break;

      case synUINT	:
	res = checkSyntax_uint(&info, self->argv[i], reason);
	break;

      case synPATH	:
	res = (checkSyntax_string(&info, self->argv[i], reason) &&
	       true);	// TODO
	break;

      case synSTRING	:
	res = checkSyntax_string(&info, self->argv[i], reason);
	break;
	
      default		:  assert(false); break;
    }
	
    SyntaxInfo_destroy(&info);

    if (!res) {
      *code = 254;
      return false;
    }
  }
  
  if (*ptr!='\0') {
    *code   = 254;
    *reason = "Not enough params given";
    return false;
  }

  return true;
}

bool
Parameters_evaluate(struct Parameters *self,
		    struct DjinniProcess const *proc,
		    unsigned int *code, char const **reason)
{
  assert(self!=0);
  assert(proc!=0);
  assert(code!=0);
  assert(reason!=0);
  
  if (self->argc==0 ||
      self->argv[0]==0) {
    *code   = 254;
    *reason = "No command given";
    return false;
  }

  if (!checkSyntax_command(self->argv[0], reason)) {
    *code   = 254;
    return false;
  }
  
  if (!DjinniProcess_chdirCfg(proc) ||
      chdir(self->argv[0])==-1) {
    *code   = 254;
    *reason = "No such command";
    return false;
  }

  if (access(".run", X_OK)!=0 && access(".new", F_OK)!=0) {
    *code   = 255;
    *reason = "Internal error: neither .run nor .new exists";
    return false;
  }

  int		fd  = open(".params", O_RDONLY);
  if (fd==-1) {
    *code   = 255;
    *reason = "Internal error: parameter syntax unspecified";
    return false;
  }

  off_t		len = lseek(fd, 0, SEEK_END);
  if (len==-1 || lseek(fd,0,0)==-1) {
    *code   = 255;
    *reason = "Internal error: failed to determine .params size";

    close(fd);
    return false;
  }

  char		buf[len+1];
  if (read(fd, buf, len)!=len) {
    *code   = 255;
    *reason = "Internal error: failed to read .params";
    
    close(fd);
    return false;
  }

  while (len>0 && buf[len-1]=='\n') --len;
  
  buf[len] = '\0';
  close(fd);
  return iterateThroughParams(self, proc, buf, code, reason);
}
