// $Id: parameters.c,v 1.2 2003/11/06 04:53:19 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "parameters.h"
#include "util-io.h"
#include "wrappers.h"
#include "util.h"

#include <assert.h>

static size_t	PARAMETERS_MAX_ARGC	=  1024;
static size_t	PARAMETERS_MAX_LEN	=  0x2000;

bool
Parameters_init(struct Parameters *self)
{
  self->argc = 0;
  self->argv = 0;
  self->lens = 0;
  self->idx_ = 0;

  self->vserver_vdir = 0;
  self->vserver_name = 0;

  return true;
}

void
Parameters_delete(struct Parameters *self)
{
  for (size_t i=0; i<self->idx_; ++i)
    free(self->argv[i]);

  free(self->argv);
  free(self->lens);

  free(const_cast(char *)(self->vserver_vdir));
  free(const_cast(char *)(self->vserver_name));
}

bool
Parameters_setArgCount(struct Parameters *self, size_t cnt)
{
  if (self->idx_!=0 || self->argc!=0) {
    assert(false);
    WRITE_MSG(2, "Parameters_setArgCount() called twice\n");
    return false;
  }

  if (cnt>=PARAMETERS_MAX_ARGC) {
      // TODO: logging
    WRITE_MSG(2, "Too much parameters given\n");
    return false;
  }

  self->lens = Emalloc(sizeof(size_t) * cnt);
  self->argv = Emalloc(sizeof(char *) * cnt + 1);
  self->argc = cnt;

    // set finish element
  self->argv[cnt] = 0;

  return true;
}

char *
Parameters_allocParam(struct Parameters *self, size_t len)
{
  if (len>=PARAMETERS_MAX_LEN) {
      // TODO: logging
    WRITE_MSG(2, "Parameters too long\n");
    return false;
  }

  if (self->idx_>=self->argc) {
    WRITE_MSG(2, "argv overflow\n");
    return false;
  }

  assert(self->argv!=0);
  
  self->argv[self->idx_] = Emalloc(sizeof(char)*len + 1);
  self->lens[self->idx_] = len;

  self->argv[self->idx_][len] = '\0';

  return self->argv[self->idx_++];
}

bool
Parameters_normalize(struct Parameters *self)
{
  if (self->idx_!=self->argc) {
    WRITE_MSG(2, "Parameters_normalizeParams can not be called without having all parameters\n");
    return false;
  }

  for (size_t i=0; i<self->argc; ++i)
    self->argv[i][self->lens[i]] = '\0';

  return true;
}
